__author__ = "Dan"


class AvatarAppearance:
    """
    The presentation aspect of an avatar.
    """

    def __init__(self, body_stroke, body_fill, eye_stroke, eye_fill):
        self.body_stroke = body_stroke
        self.body_fill = body_fill
        self.eye_stroke = eye_stroke
        self.eye_fill = eye_fill
