# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2019-04-18 07:28
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("aimmo", "0009_add_game_status")]

    operations = [
        migrations.AlterField(
            model_name="game",
            name="auth_token",
            field=models.CharField(blank=True, max_length=24),
        )
    ]
