# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-08-28 10:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("aimmo", "0007_auto_20180515_1300")]

    operations = [
        migrations.AlterField(
            model_name="game", name="public", field=models.BooleanField(default=False)
        )
    ]
